( function () {
    /*
     * SDE comment module
     * 
     * 
     * @TODO Use _baseConfig / SDE.utils.Config
     * @TODO remove 'lastmodified'
     * @TODO cleanup
     * 
     */
    SDE.namespace( 'widget.Comments' );

    var root,       // div-root, usually #comments
    path,           // article path, sth like '1.1112650'
    token,          // one-time-token, gets initially delivered by the commentform, updated on successful POST
    ts = 0,         // timestamp of latest activity, invalidates varnish cache
    baseUrl = location.protocol + '//' + location.hostname + '/app/service/integration/index.php/comment/',
    referer = '',   // the page we are actually on
    mode = 'UN_MODERATED',
    fbAuthentication,
    fbPermissions = {},
    panel = 'all',
    authType = [],
    page = 1;
    
    SDE.widget.Comments = function ( userConfig ) {
        SDE.log.group( "Comments" );
                
        root  = SDE.$('#comments'); 
        path  = userConfig.path;
        mode  = userConfig.mode;
        referer = root.data('referer');
        panel = root.data('panel');
        page = root.data('page');
        lastmodified=0; // async ajax call sometimes is slower than the         

        // *****
        // ***** CORE FUNCTIONALITY 
        // *****

        /**
         * Submit a comment form. We always supply our current token, if the call
         * validates (not meaning that is really successful) we receive a new
         * token.
         */
        SDE.$('form', root).live('submit', function(ev){    
            ev.preventDefault();
            var form = SDE.$(this),
            button = SDE.$('input[type=submit]', form),
            textarea = SDE.$('textarea[name=body]:first', form),
            data = form.serializeArray();
            data.push({
                name:'path', 
                value:path
            });
            data.push({
                name:'status', 
                value:mode
            });
            
            // check that text is not too short and not default text
            if (textarea.val() == textarea.data('default') || textarea.val().length <5) {
                _notify(form, 'Bitte schreiben Sie zun&auml;chst Ihren Beitrag in das Textfeld.');
                return false;
            }
            
            // Post message on FB, too? Then check auth status first. Actual posting
            // will be done from the backend then.
            if( form.find('input[name=post_facebook]:first').attr('checked') ){
                if(!_sdeAuthenticated() && !fb.isConnected()){
                    _showLoginSelector();
                    return false;
                }
                if(!fb.isConnected() || !fb.canPublish()){
                    fb.showLogin('publish_stream');
                    return false;    
                }
            }
            
            // Post message on SDE    
            SDE.$.ajax({                
                url: baseUrl + form.data('action'),
                type: 'post',
                data: data,
                error: function(xhr, type){
                    if(xhr.status == 401){
                        _showLoginSelector();
                    }
                },
                success: function(d){
                    lastmodified = d.lastmodified;          // Update the last modified timestamp for the article (used with subsequent AJAX GETs)
                    token = d.token;    // Update the token for POST requests
                    /* We display the result differently depending on:
                     * - the type of form sent
                     * - article mode
                     *      MODERATED: We display a thank you message
                     *      UN_MODERATED: Instant user feedback is provided
                     * 
                     * Instant user feedback:
                     * - comments: we load the maximum panel page. The comment 'should' be there then
                     * - answers: we replace the form with the rendered answer
                     */                  
                    if(mode == 'UN_MODERATED'){
                        switch(form.data('type')){
                            case "comment":
                                form.replaceWith(tpl.msg(d.message));
                                _load('all', 999);
                                // Try to scroll to the correct comment
                                SDE.$(window).scrollTop(SDE.$('#commentPanel div.comment:last').offset().top);
                                break;
                            case "answer":
                                var id = 'comment_answer_' + SDE.uid(),
                                a = tpl.answer(d.comment),
                                t = SDE.$('<div id="' + id + '" class="comment answer" style="display:none;"><ol>' + a + '</ol></div>');
                                form.parent().before(t);
                                form.parent().fadeOut().remove();
                                t.fadeIn();
                                break;
                            default:
                                form.replaceWith(tpl.msg(d.message));
                                break;
                        }
                    } else {
                        form.replaceWith(tpl.msg(d.message));
                    }
                },
                beforeSend: function(){
                    button.attr('disabled', true).css('border', '#666').css('background', '#999');
                },
                complete: function(){
                    button.attr('disabled', false).css('border', '#922').css('background', '#B70000');
   
                }
            });
            return false;
        });

        /**
         * Chars remaining calculation, restricting to data-max
         */
        SDE.$('textarea', root).live({
            'keyup': function(){
                var field = SDE.$(this),
                max = field.data('max'),
                remaining = parseInt( max - field.val().length);
                if (remaining <= 0 ){
                    remaining = 0;
                    field.val(field.val().substr(0,max));
                }
                field.parent().find('.charsremaining').html(remaining);
            },
            'focusin': function(){
                if(SDE.$(this).val() == SDE.$(this).data('default')){
                    SDE.$(this).val( '' );
                    SDE.$(this).css('color','#666');
                }
            },
            'focusout': function(){
                if( SDE.$(this).val() == '' ){
                    SDE.$(this).val( SDE.$(this).data('default') );
                    SDE.$(this).css('color','#999');
                }
            }
        });

        /**
         * Switching panels (editor, user, all)
         */
        SDE.$( '.menu li:not(.current)', root ).live('click', function() {
            SDE.$( '#comments .menu li' ).removeClass('current');
            SDE.$(this).addClass('current');
            _load( SDE.$(this).data('show'), 1, false );  // Load page 1 from the chosen panel
        });

        /**
         * Paging within a panel if necessary
         */
        SDE.$( 'div.paging li[data-page]:not(.current)', root ).live('click', function() {
            var page = SDE.$(this).data('page'),
            mode = SDE.$( '#comments .menu li.current' ).data('show');
            _load(mode, page, false); // Load page # from the chosen panel  
        });

        /**
         * Recommend SDE
         */
        SDE.$( '.comment .functions li.recommend', root).live('click', function() {
            var comment = SDE.$(this).closest('.comment');
            SDE.$.ajax({
                url: baseUrl + 'extendedFunctionality/',
                type: 'post',
                data: {
                    id: comment.data('id'),
                    path: path,
                    what: 'likeSDE'
                },
                error: function(xhr, type){
                //_notify(comment, xhr);
                },
                success: function(d){
                    // TODO do we want to reflect that change by invalidating the cache immediately?
                    if( d.lastmodified ) lastmodified = d.lastmodified;
                    token = d.token;
                    SDE.$('.functions > p', comment).html(tpl.like(d.like_sde, d.like_fb));
                    if ( comment.find(".recommend").find('span').length == 0){
                        comment.find(".recommend").find('img:first').after("<span>" + d.like_sde + "</span>");
                    } else {
                        comment.find(".recommend").find('span').replaceWith("<span>" + d.like_sde + "</span>");
                    }
                }
            });
        });

        /**
         * Initialisation - this is bound to the 'facebook:ready' event,
         * which is triggered upon completed initialisation of the Facebook API.
         */
        SDE.$(document).bind('facebook:ready', function(e){

            // We track creation and deletion of edges (aka like).
            FB.Event.subscribe('edge.create', function(e){
                _fbEdge('create', e);
            });
            FB.Event.subscribe('edge.remove', function(e){
                _fbEdge('remove', e);
            });
            FB.Event.subscribe('auth.authResponseChange', function(resp){
                fb.auth(resp);
            });
            FB.getLoginStatus(function(resp){
                fb.auth(resp);
            });
            
        });


        /**
         * Squeal and answer functionality for comments. 
         */
        SDE.$( 'div.comment:not(.answer) .functions li', root ).live('click', function() {
                
            // expand the right snippet
            var action    = SDE.$(this),
            isActive  = action.hasClass('active'),
            comment   = action.closest('.comment'),
            commentId = comment.data('id');
                
            // I couldn't find the right jquery syntax for selecting every comment that
            // is not an answer and has functions links for squeal/answer. Stupid me.
            if( !(action.hasClass('squeal') || action.hasClass('answer')) ){
                return;
            }


            // An action has been triggered on a comment that is either
            // - not currently the currently selected one
            // - is already selected and it's the same action for the second time
            // We collapse all open comments.
            SDE.$('div.comment.select', root).each( function(){
                _collapse(this);
            });

            if( !isActive ){
                comment.addClass('select');
                comment.next( '.comment:first' ).addClass( 'next' );
                action.addClass('active');

                if( SDE.$(this).hasClass('squeal') ){
                    comment.after(tpl.squealForm(commentId));
                //                    comment.after('<div class="answer_top_marker"></div>');
                }
                if( SDE.$(this).hasClass('answer') ){
                    comment.after(tpl.answerForm(commentId));
                    SDE.$('#answer_text').val(SDE.$('#answer_text').data('default'));
                    if( SDE.$(this).data('count') > 0 ){
                        SDE.$.ajax({
                            url: baseUrl + 'answers',
                            data: {
                                'path':path, 
                                'id':commentId, 
                                'lastmodified':lastmodified
                            },
                            success: function(d){
                                var t = '',
                                count = 0;
                                id = 'comment_answer_' + SDE.uid();
                                SDE.$.each(d.answers, function(idx, ans){
                                    t += tpl.answer(ans);
                                    count++;
                                });
                                t = '<div id="' + id + '" class="comment answer"><span class="bubble"></span><div class="answer_header">' + count + ' '+ tpl.plural('Antwort', 'Antworten', count) + '</div><ol>' + t + '</ol></div>'; 
                                comment.after(t);
                                SDE.$('#comments').trigger('_loadAnswers');
                            }
                        });
                    }
                    
                    
                }
                comment.nextUntil('.comment.next:first').fadeIn('slow','linear');
            }

        });

        // Squeal for answers. They open a section that can be toggled.
        SDE.$( 'div.comment.answer .functions li.squeal', root).live('click', function(ev) {
            var action   = SDE.$(this),
            answer   = action.closest('li.answer'),
            isActive = action.hasClass('active');

            // Fade out all other squeals
            answer.parent().find('.comment.squeal').fadeOut().remove();
            answer.find('.functions li.active').removeClass('active');

            if( !isActive ){
                action.addClass('active');
                var form = SDE.$(tpl.squealForm(answer.data('id')));
                answer.after(form);
                form.fadeIn();
            }
        }); 
     
        
        // delete for comments
        SDE.$('div.comment:not(.answer) .functions li.delete', root ).live('click', function() {
            SDE.$(root).data('comment-delete', SDE.$(this).closest('.comment'));
            _confirmDelete.show();
        });


        //delete for answers. They open a section that can be toggled.
        SDE.$( 'div.comment.answer .functions li.delete', root).live('click', function() {
            SDE.$(root).data('comment-delete', SDE.$(this).closest('li.answer'));
            _confirmDelete.show();

        });
        
        // ************** confirm box *****************
        // close/cancel
        SDE.$('#comments_confirm .close,.cancel').live('click', function() {
            _confirmDelete.close();
        });

        // ok
        SDE.$('#comments_confirm .ok').live('click', function() {
            _confirmDelete.close();
            var commentId = SDE.$(root).data('comment-delete');
            _deleteComment(commentId);
        });
        
        
   
        
        var _confirmDelete = {
            'show': function(commentId){
                var confirmBox =
                '<div id="comments_confirm" class="lightbox" role="dialog" aria-labelledby="comments_confirm" tabindex="-1">' +
                '  <div class="body">' +
                '    <p>Sind Sie sicher, dass Sie diesen Beitrag l&ouml;schen m&ouml;chten?</p>' +
                '    <button class="cta cancel">Abbrechen</button>' +
                '    <button class="cta ok">Ok</button>' +
                '    <button class="close">Schlie&szlig;en</button>' +
                '  </div>' +
                '</div>'
                ;
                var panelConfirm = '#comments_confirm',
                panelShime = '#comment_shime';
                var shimeBox = '<div id="comment_shime" class="shimebox"></div>';
                SDE.$( 'body' ).append( confirmBox ).append( shimeBox );
                // Set position for loginbox
                setBoxPositioning( panelConfirm, panelShime );
                // Show loginbox
                SDE.$( panelShime ).fadeIn( 'slow', function() {
                    SDE.$( panelConfirm ).fadeIn( 'slow' );
                } );
            },
            'close': function(){
                SDE.$('#comments_confirm').fadeOut(function(){
                    SDE.$('#comment_shime').fadeOut().remove(); 
                }).remove();
            }
        }


        function _deleteComment(comment){
            SDE.$.ajax({
                url:  baseUrl + 'delete',
                type: 'POST',
                data: {
                    id: comment.data('id'),
                    'path': path,
                    'lastmodified': lastmodified
                },
                error: function(d){
                //_notify(comment, d);  
                },
                success: function(d){
                    if (comment.hasClass('answer')) {
                        comment.replaceWith('<li class="answer">'+tpl.msg(d.message)+'</li');
                    } else if (comment.hasClass('comment') && d.message){
                        comment.replaceWith('<div class="comment">'+tpl.msg(d.message)+'</div>');
                        SDE.$('.comment.answer').slideUp();
                        SDE.$('.discuss.answer').slideUp();
                    } else {                                                    
                        comment.slideUp();
                        SDE.$('.comment.answer').slideUp();
                        SDE.$('.discuss.answer').slideUp();
                    }                        
                }
            });
        }
        // END of delete comments
        

        // Generic loading.
        
        function _load(panel, page, ajaxLoad){
            if(typeof panel == 'undefined'){
                panel = 'all'
            };
            if(typeof page == 'undefined'){
                page = 0
            };
            if(typeof ajaxLoad == 'undefined'){
                ajaxLoad = true
            };            
            if(ajaxLoad){
                SDE.$.ajax({
                    url: baseUrl + 'list',
                    data: {
                        'path':path, 
                        'mode':panel, 
                        'page':page,
                        't':lastmodified
                    },
                    success: function(d){
                        var newPanel = tpl.panel(d.mode, d.count, d.pager);
                        SDE.$.each(d.comments, function(idx, comment){
                            newPanel += tpl.comment(comment);
                        });
                        SDE.$('#commentPanel').fadeOut('fast').remove();
                        SDE.$('#comments').append('<div id="commentPanel">' + newPanel + tpl.pager(d.pager) + '</div>').fadeIn('slow');
                        /*
                        * Trigger the '_load' event. We use this to modify comments for
                        * admins/editors that will receive additional JS upon page load
                        * allowing them to mark comments as valuable
                        */
                        SDE.$('#comments').trigger('_load');
                    }
                });
            } else {
                var target = tpl.commentPath(panel, page);
                window.location.href = target;
            }
        }

        /**
         * Panel has been loaded, process things
         */
        SDE.$('#comments').bind('_load _loadAnswers', function(e){
            var type = (e.type == '_load') ? '#commentPanel div.comment:not(.answer)' : '#commentPanel div.comment.answer li.answer';
            SDE.$(type, root).hover(
                function() {
                    var userId = SDE.$(this).data('user-id'),
                    type = SDE.$(this).data('type'),
                    show = _showDelete(userId, type)
                    ;
                    if(show == true){
                        SDE.$(".functions.squealOrDelete li.delete", this).css('display', 'inline');
                    }else{
                    //alert('noshow uid: ' + userId + '   fb: ' + fb.userID() + '  t:' + type);
                    }
                    SDE.$(".functions.squealOrDelete", this).show();
                },
                function() {
                    SDE.$(this).find(".functions.squealOrDelete").hide();
                }
                );
        });
       
       
        /**
         * Get the timestamp of the last modification for
         * this article. Then load initial set of comments and populate the user
         * section
         */
        SDE.$.ajax({          
            url: '/app/service/integration/lastmod.php',
            type: 'GET',
            data: {
                'path' : path
            },
            success: function(d){
                lastmodified = d.lastmodified;
                _load( panel, page );
                _getUser();
            }
        });

        // *****
        // ***** ADDITIONAL PANELS  *********************************
        // *****

        SDE.log.groupEnd();

    };
    // END OF  SDE.widget.Comments = function ( userConfig )



    var fb = {
        'auth': function(fbResp){
            fbAuthentication = fbResp;
            _getUser();
            if(fb.isConnected()){
                FB.api('/me/permissions', function(resp){
                    if(SDE.$.isArray(resp.data)){
                        fb.perms(resp.data[0]);
                    }
                })
            }
        },
        'perms': function(fbResp){
            fbPermissions = fbResp;
        },
        'isAuthenticated': function(){
            return ( (typeof fbAuthentication != 'undefined') && (fbAuthentication.status != 'unknown') );
        },
        'isConnected': function(){
            return ( (typeof fbAuthentication != 'undefined') && (fbAuthentication.status == 'connected') );
        },
        'canPublish' : function(){
            return (fb.isConnected() && (typeof fbPermissions['publish_stream'] !== 'undefined') && fbPermissions['publish_stream'] === 1);
        },
        'userID': function(){
            if( fb.isConnected() ){
                return fbAuthentication.authResponse.userID;
            }
            return null;
        },
        'showLogin': function(perms){
            FB.login(function(response){               
                }, {
                    scope : perms
                } );   
        }
    }
    
    function _showDelete(userId, type){
        var result = 
        (type=='sde' && _sdeAuthenticated() && (userId == _getCookie('suid2')) )
        ||
        (type=='facebook' && fb.isConnected() && (userId == fb.userID()) )
        ;
        return result;
    }

    function _notify(o, msg){
        SDE.$('div.notify', o).attr('class', 'notify').html(msg).show().delay(2000).fadeOut('slow');
    }

    /**
     * Collapse a comment
     */
    function _collapse(comment){
        var c = SDE.$(comment);
        c.nextUntil('.comment.next').remove();
        c.removeClass('select');
        c.next('.comment.next').removeClass( 'next');
        c.find('.functions li.active').removeClass('active');
    }


    /**
     * Track Facebook likes in our internal database
     */
    function _fbEdge(action, url){
        SDE.log('FB edge.%s: %s', action, url);
        SDE.$.ajax({
            url: baseUrl + 'likeFB',
            type: 'post',
            data: {
                edge: action, 
                url: url,
                path: path
            },
            success: function(d){
                // TODO do we want to reflect that change by invalidating the cache immediately?
                if( d.lastmodified ) lastmodified = d.lastmodified;
                SDE.log("FB edge.%s result: %o", action, d);
                var comment = SDE.$('div.comment[data-id=' + d.id + ']', root);
                SDE.$('.functions > p', comment).html(tpl.like(d.like_sde, d.like_fb));
            }
        });
    }
                

    /**
     * Approximately determine the currently logged in user. Uses the sdelogin
     * cookie for SDE, for FB
     * only if the settings panel has been seen already (avoid calls to backend).
     */
    function _getUser(){
        var profile = SDE.$("div.discuss strong:first ~ span.profile", root);

        if(_sdeAuthenticated()){
            profile.replaceWith('<span class="profile sde" >Mein Profil</span>');
        } else if (fb.isConnected()) {
            profile.replaceWith('<span class="profile facebook"><fb:name useyou="false" uid=loggedinuser/></span>');
            FB.XFBML.parse(document.getElementById('comments'));
        }
        else {
            profile.replaceWith('<span class="profile none" >Anmelden</span>');
        }

    }
    

    function _commentUser(user){
        if ( user == undefined || user.type == false ) {
            SDE.log("User is not authenticated");
            tpl.commentUser('unbekannter Benutzer');
        } else {
            if ( user.type == 'sde' ){
                SDE.log("User is a SDE user");
                username = '<b>' + user.sde.username + '</b>';
                picture = '<img src="' + user.sde.picture + '">';
                tpl.commentUser(username, picture);
            }else if ( user.type == 'facebook' ){
                SDE.log("User is a Facebook user");
                username = '<span class="icon">' + user.facebook.username + '</span>';
                picture += '<a href="https://www.facebook.com/' + user.facebook.picture + '" ><img src="https://graph.facebook.com/' + user.facebook.picture + '/picture?type=square" height="75"/></a>';    
                tpl.commentUser(username, picture);
            }
        }
    }


    /**
     * Read a cookie's value.
     * 
     * SDE.$.cookies.get does not decode '+' characters
     */
    function _getCookie(c_name){
        var i,x,y,ARRcookies=document.cookie.split(";");
        for (i=0;i<ARRcookies.length;i++){
            x=ARRcookies[i].substr(0,ARRcookies[i].indexOf("="));
            y=ARRcookies[i].substr(ARRcookies[i].indexOf("=")+1);
            x=x.replace(/^\s+|\s+$/g,"");
            if (x==c_name){
                return unescape(y.replace(/[+]/g," "));
            }
        }
    }
    
    function _sdeAuthenticated(){
        return (_getCookie('sdelogin') === '1');
    }
    
    
    /**
     * Lightbox functionality for authentication
     */
    SDE.$('span.profile', root).live('click', function(){
        var clicked = SDE.$(this);
        if( clicked.hasClass('none') ){
            // Show login selector
            _showLoginSelector();
        } else if ( clicked.hasClass('sde') ){
            // Show SDE-Loginbox, can navigate to myData from there
            _showSdeLogin();
        }

    });
    
    
    /**
     * Show selector, positioned in the middle of the screen
     */
    var _showLoginSelector = function (){
        var selectorBox = 
        '<div id="comments_login_selector" class="lightbox" role="dialog" aria-labelledby="comments_login_selector">' +
        '  <div id="comments_login_select">' +
        '    <h3>Sie sind noch nicht angemeldet</h3>' +
        '      <ul class="user_login">' +
        '        <li class="sz">Mit S&uuml;ddeutsche.de-Login anmelden</li>' +
        '        <li class="fb">Mit Facebook anmelden</li>' +
        '      </ul>' +
        '  </div>' +
        '  <button class="close">Schlie&szlig;en</button>' +
        '</div>';
        var shimeBox = '<div id="comments_shime" class="shimebox"></div>',
        panelShime = '#comments_shime',
        panelSelector = '#comments_login_selector';
        SDE.$( 'body' ).append(selectorBox).append(shimeBox);
        setBoxPositioning( panelSelector, panelShime );
        SDE.$( panelShime ).fadeIn( 'slow', function() {
            SDE.$('#comments_login_selector').fadeIn();
        });
    }
    
    /**
     * Click in the selector
     * - close the selector
     * - show sde login
     * - show FB login
     */
    SDE.$('#comments_login_selector .sz,.fb,.close',root).live('click', function(){
        var clicked = SDE.$(this);
        SDE.$('#comments_login_selector').remove();
        SDE.$('#comments_shime').remove();
        if( clicked.hasClass('sz') ){
            _showSdeLogin();
        } else if( clicked.hasClass('fb') ){
            fb.showLogin();
        }
    });
    
    /**
     * Calculate the correct offset. Taken from SDE.widget.Login.
     */
    var setBoxPositioning = function( loginObj, shimeObj ) {
        var win   = SDE.$( window ),
        doc   = SDE.$( document ),
        obj   = SDE.$( loginObj ),
        shime = SDE.$( shimeObj );
        obj.css( {
            'left' : Math.max( 0, ( ( win.width() - obj.width() ) / 2 ) + win.scrollLeft() ) + 'px',
            'top'  : Math.max( 0, ( ( win.height() - obj.height() ) / 2 ) + win.scrollTop() ) + 'px'
        } );
        
        shime.css( {
            'width'  : Math.max( win.width() ) + 'px',
            'height' : Math.max( doc.height() ) + 'px'
        } );
        
    };
    
    
    /**
     * Popup SDE login dialog
     * 
     * TODO set the correct link
     */
    var _showSdeLogin = function(){
        SDE.widget.Login.init({
            type: 'comments'
        });
        // Dirty. We pass the referer URL manually. Hate it.
        SDE.$('#loginbox iframe').load(function(){
            SDE.$('#loginbox iframe').contents().find('a').each(function(){
                SDE.$(this).attr('href', _replaceParam(SDE.$(this).attr('href'), 'returnto', referer));
            });
        });
    }
    
    /**
     * Replace link
     */
    var _replaceParam = function(s, key, val){
        var start = s.indexOf('?');
        if(start!=-1){
            var qp = {}, qs = s.slice(start+1), re = /([^&=]+)=([^&]*)/g, m;
            while (m = re.exec(qs)) {
                qp[decodeURIComponent(m[1])] = decodeURIComponent(m[2]);
            }
            qp[key] = val;
            return (s.slice(0,start) + '?' + SDE.$.param(qp));
        }
        return s;
    }



    var tpl = {
        'commentPath': function(panel,page,id){
            if(typeof panel == 'undefined') panel = '';
            if(typeof page == 'undefined') page = '';
            if(typeof id == 'undefined') id = '';
            return referer + '?commentspage=' + panel + ':' + page + ':' + id + '#comments';
        },
        'fbLike': function(c){
            return '<fb:like href="' + baseUrl +'fbArticle?id=' + c.id + '&path=' + path + '" send="false" layout="button_count" width="115" show_faces="false" action="recommend" font="arial"></fb:like>';
        },
        'squealForm': function(commentId){
            var t = '<div class="comment squeal"><span class="bubble"></span><form class="squeal_form" data-action="squeal" data-type="squeal"><div class="notify"></div>' + 
            '<label for="squeal_text" class="squeal_text">Sie sind der Meinung, dass dieser Beitrag entfernt werden sollte? ' +
            'Dann schreiben Sie uns bitte hier eine kurze Nachricht:' +
            '<textarea name="body"></textarea></label>' +
            '<input type="hidden" name="id" value="' + commentId + '">' +
            '<input type="submit" class="cta arrow" value="Abschicken"></form></div>';
            return t;
        },
        'answer': function(a){
            var t = '<li class="answer" data-id="'+ a.id +'" id="comment_'+a.id+'" data-type="' + a.type+ '" data-user-id="' + a.user_id + '">';
            if(a.type == 'sde'){
                if (!a.picture.match("anonymous_small.gif$")) {
                    t += '<div class="teaser" style="min-height:75px">';
                    t += '<a href="http://suedcafe.sueddeutsche.de/' + a.username + '" ><img src="' + a.picture + '" class="profile"></a>';
                    t += '<a href="http://suedcafe.sueddeutsche.de/' + a.username + '" ><strong>' + a.username + '</strong></a>';
                } else {
                    t += '<div class="teaser">';
                    t += '<strong>' + a.username + '</strong>';
                }
            } else {
                t += '<div class="teaser"> <a href="https://www.facebook.com/' + a.picture + '" target="_blank"><img src="https://graph.facebook.com/' + a.picture + '/picture?type=square" height="75"/></a>';
                t += '<strong><a href="https://www.facebook.com/' + a.picture + '" target="_blank">' + a.username + '</a></strong>';
            }
            t += tpl.prettyDate(a.created_at) + tpl.squealOrDelete(a) +
            '<p class="entry-summary">' + tpl.nl2br(a.body) + '</p><div class="notify"></div></li>';
            return t;
        },
        'answerForm': function(commentId){
            if(mode.match('NO_POSTING')){
                return '';
            }
            var t = '<div class="discuss answer"><span class="bubble"></span><form data-action="add" data-type="answer" class="answer_form">' +
            '<label for="answer_text" class="answer_text"><em>noch <span class="charsremaining" aria-live="off">2500</span> Zeichen</em> ' +            
            '<textarea name="body" aria-controls="charsremaining" data-default="Ihre Antwort ..." data-max="2500" data-action="answer">Ihre Antwort ...</textarea>' +
            '</label><div class="notify"></div> <label for="answer_facebook" class="answer_facebook icon"> ' +
            '<span class="offscreen">Diesen Beitrag</span> auch auf Facebook posten <input type="checkbox" name="post_facebook" value="0">' +
            '</label><input type="hidden" name="id" value="' + commentId + '" ><input type="submit" class="cta arrow" value="Antworten">' + 
            '</form>'+
            '</div>';
            return t;
        },
        'squealOrDelete': function(c){
            var t = '<div class="functions squealOrDelete"><ul><li class="delete" data-name="' + c.username + '" data-id="'+c.id+'">L&ouml;schen</li>'+
            '<li class="squeal"> Melden </li></ul></div>';
            return t;
        },
        'nl2br' : function(t){
            return t.replace(/(\r?\n){2,}/g, "<br/><br/>").replace(/(\r\n)/g, "<br/>");
        }, 
        'comment': function(c){            
            var t = '<div class="' + (c.editor_pick == true ? "comment editor" : "comment" ) + '" data-id="'+c.id+'" id="comment_'+c.id+'" data-type="' + c.type+ '" data-user-id="' + c.user_id + '"><div class="teaser">';
            if(c.type == 'sde' && c.suedcafe_user == 1){
                if (!c.picture.match("anonymous_small.gif$")) {
                    t += '<a href="http://suedcafe.sueddeutsche.de/' + c.username + '" ><img src="' + c.picture + '" class="profile"></a>';
                } 
                t += '<strong><a href="http://suedcafe.sueddeutsche.de/' + c.username + '" >' + c.username  + (c.is_editor == true ? ", S&uuml;ddeutsche.de" : "") +'</a> </strong> ';
            } else if  (c.type == 'sde') {
                if (!c.picture.match("anonymous_small.gif$")) {
                    t += '<img src="' + c.picture + '" class="profile">';
                } 
                t += '<strong>' + c.username + (c.is_editor == true ? ", S&uuml;ddeutsche.de" : "")  + '</strong> ';
            }
            else {
                t += '<a href="https://www.facebook.com/' + c.picture + '" target="_blank"><img src="https://graph.facebook.com/' + c.picture + '/picture?type=square" height="75"/></a>';    
                t += '<strong><a href="https://www.facebook.com/' + c.picture + '" target="_blank">' + c.username  + '</a></strong> ';
            }
            t += tpl.prettyDate(c.created_at) + '  ' + '<span style class="editorpick '+ ( (c.editor_pick == true) ? 'icon active':'') + '"></span>';
            t += tpl.squealOrDelete(c);
            t += '<p class="entry-summary">' + tpl.nl2br(c.body) + '</p></div><div class="functions"><p>' + tpl.like(c.like_sde, c.like_fb, c.editor_pick) + '</p>' +
            '<ul><li class="recommend"> <img src="/app/service/integration/images/Lesenswert_2.png" />';
            if (c.like_sde > 0) {
                t += '<span> ' + c.like_sde + ' </span>';
            }
            t += '</li>';    
                     
            
            if (c.answer_count >0 || !mode.match('NO_POSTING')) {
                t += '<li class="answer" data-count="' + c.answer_count + '">' ;
                if (c.answer_count >0 ){ 
                    t += c.answer_count + ' ' + tpl.plural('Antwort' , 'Antworten', c.answer_count);
                } 
                if (c.answer_count >0 && !mode.match('NO_POSTING')) {
                    t += "&nbsp;&nbsp;&nbsp;&nbsp;";
                }
                if (!mode.match('NO_POSTING')){
                    t += ' <span> Antwort schreiben</span>'
                }
                t += '</li>';
            }
            t += '</ul></div><div class="notify"></div></div>';
            return t;
        },
        'like': function(sde, fb, editor_pick){
            var total = sde+fb;            
            if(total==0 && editor_pick != 'true') return '';
            
            var startText = '';
            if (total > 0 && editor_pick == 'true') {
                startText = total + ' Leser und die Redaktion empfehlen';
            } else if (total > 0 && editor_pick != 'true') {
                startText = total+' Leser '+tpl.plural('empfiehlt','empfehlen',total);
            } else {
                startText = 'Die Redaktion empfiehlt';
            }
            
            return startText+' diesen Beitrag';
        },
        'msg': function(msg){
            return '<div class="comment_msg"><p>'+msg+'</p></div>';
        },
        'plural': function(s,p,c){
            if(c==1) return s;
            return p;
        },
        'prettyDate': function(ts){
            var pretty = "",
            clazz = "datetime",
            date = new Date(ts*1000),
            now = (new Date()).getTime(),
            diff = ((now - date.getTime()) / 1000),
            day_diff = Math.floor(diff / 86400);
            if ( isNaN(day_diff) || day_diff < 0 ) return '';
            if( day_diff > 0 ){
                pretty = date.getDate() + '.' + (date.getMonth()+1) + '.' + date.getFullYear() + ' | ' + date.getHours() + ':' + ((date.getMinutes() < 10) ? '0':'') + date.getMinutes() + ' Uhr';
            } else if (diff < 120) {
                clazz += " red";
                pretty = "Gerade eben";
            } else if (diff < 1800) {
                pretty = 'vor ' + Math.floor(diff/60) + ' Minuten';
            } else if (diff < 3600) {
                pretty = "In der letzten Stunde";
            } else {
                var hDiff = Math.floor(diff/3600);
                pretty = 'vor ' + hDiff + tpl.plural(' Stunde', ' Stunden', hDiff);
            }
            return '<span class="'+clazz+'">'+pretty+'</span>';
        },
        'panel': function(mode, count, pager){
            SDE.log("mode: %o, count: %o, pager: %o", mode, count, pager);
            var t = "",
            menu = "",
            panel = "",
            menuEntries = 1,
            menuWidth = 560;
            if(count.all > 0){
                // We have to scale menu entries if not all are available. Available width is 536px
                if( count.editor > 0 ) menuEntries++;
                if( count.user > 0 ) menuEntries++;
                menuWidth = (menuWidth - (menuEntries+1)) / menuEntries;
                
                // Build menu
                var firstIsTaken = false;
                menu += '<div class="menu"><ul>';
                if( count.editor > 0 ){
                    menu += '<li class="best ' + ((firstIsTaken) ? '' : 'first') +' '+ ((mode=='editor') ? 'current':'') + '" style="width:' + Math.floor(menuWidth) +
                    'px;" data-show="editor">Unsere Empfehlungen <span class="bubble icon"></span></li>';
                    firstIsTaken = true;
                }
                if( count.user > 0 ){
                    menu += '<li class="user ' + ((firstIsTaken) ? '' : 'first') +' '+ ((mode=='user') ? 'current':'') + '" style="width:' + Math.floor(menuWidth) +
                    'px;" data-show="user">' + count.user + tpl.plural(' Leserempfehlung', ' Leserempfehlungen', count.user) + '<span class="bubble icon"></span></li>';
                    firstIsTaken = true;
                }
                menu += '<li class="all last ' + ((firstIsTaken) ? '' : 'first') +' '+ ((mode=='all') ? 'current':'') + '" style="width:' + Math.ceil(menuWidth) +
                'px;" data-show="all">' + (count.all > 1 ? 'Alle '+count.all+' Beitr&auml;ge' : ' 1 Beitrag') + '<span class="bubble icon"' + (menuEntries==1 ? 'style="left:270px;"' : '') +' ></span></li>';
                menu += '</ul></div>';
                t += menu + tpl.pager(pager, mode);
            }
            return t;
        },
        'pager' : function(pager, mode){           
            // Build panel/pager
            // Calculate available page numbers
            var pagerDelta =2,
            minsurplus = (pager.current <= pagerDelta) ? (pagerDelta - pager.current + 1) : 0,
            maxsurplus = (pager.current >= (pager.last - pagerDelta)) ? (pager.current - (pager.last - pagerDelta)) : 0,
            pages = {
                'min' : Math.max(pager.current - pagerDelta - maxsurplus, 1),
                'max' : Math.min(pager.current + pagerDelta + minsurplus, pager.last)
            },
            panel = '<div class="panel"><div class="paging first"><span>';
            switch(mode){
                case 'editor':
                    panel += 'Die Redaktion hat ' + pager.count + ' ' + tpl.plural("Beitrag", "Beitr&auml;ge", pager.count) + ' ausgew&auml;hlt';
                    break;
                case 'user':
                    if(pager.count > 1){
                        panel += 'Die '  + pager.count + ' von Lesern meistempfohlenen Beitr&auml;ge:';
                    }
                    break;
                case 'all':
                    break;    
                default:
                    panel += '<!-- Unknown panel mode: ' + mode + ' -->';
            }
            panel += '&nbsp;</span><ol><li> Seite </li>';
            var i = pages.min;
            if( pages.min != 1){
                panel += '<li data-page="1">1 .. </li>';
            }
            for(i; i < pages.max; i++){
                panel += '<li data-page="'+i+'" class="'+ ((i==pager.current) ? 'current':'') +'"> '+i+' </li>'
            }
            if( pages.max == pager.last ){
                panel += '<li data-page="'+i+'" class="last '+ ((i==pager.current) ? 'current':'') +'"> '+i+' </li>'
            }else{
                panel += '<li data-page="'+i+'" class="'+ ((i==pager.current) ? 'current':'') +'"> '+i+' </li>'
                panel += '<li data-page="'+pager.last+'" class="last '+ ((pager.last==pager.current) ? 'current':'') +'"> .. '+ pager.last +' </li>'
            }
            panel += '</ol></div></div>';
            
            return panel;
        }
    };

}() );
